#' Differential abundance analysis for zero-inflated data.
#'
#' This function processes the two group differential abundance (DA) analysis for zero-inflated data.
#' It has the flexibility to let users to choose various detection limits (DL). The function returns
#' estimates on group means, group biological point mass value (BPMV) proportions, standard deviation,
#' and p-values.
#'
#' @param indata Specifies the input data matrix. Rows are features and columns are subjects.
#' @param cov.matrix A matrix for variables. Categorical data should be represented by dummy variables
#' contain only 0 and 1.
#' @param test_cov A vector corresponding to the coloums in \code{cov.matrix} which specify the
#' variables to be tested.
#' @param test_cov_conti A vector corresponding to the coloums in \code{cov.matrix} which specify the
#' continuous variables.
#' @param min.non0n The minimum number of nonzero observations for features to be included in the analysis.
#' Default and the minimum value is 3. Please input a number equal or larger than 3.
#' @param requiredn The minimum number of nonzero obs while getting prior distribution of variance.
#' Default value is 10.
#' @param requiredn2 The minimum number of features while getting prior distribution of variance.
#' Default value is 30.\cr
#' If requiredn is specified, it will retrun features used in getting prior distribution of variance.
#' If the number of features used is less than requiredn2, the function returns top requiredn2
#' features with the smallest all point mass value (PMV) proportions.
#' @param DL_method Specifies the detection limit method. The options are:\cr
#' Fixed Difference (default): for each feature, DL is the minimun value for all nonzero observation
#' minus a number sepcified by DL_value\cr
#' Fixed Rate: for each feature, DL is the minimun value for all nonzero observation devided by
#' a number sepcified by DL_value\cr
#' Fixed Value: DL is the same value (a number sepcified by DL_value) for all features.
#' @param DL_value Custom specified number. Default is 0.1 for "Fixed Difference".
#' @param maxit_MLE The maximum number of iterations while re-estimating the model parameters.
#' The default is 100.
#' @param maxit The maximum number of iterations applied to the optimization function.
#' @param test_model A vector of models to be tested. The default value is c("Both","Mean","Pzero").\cr
#' The  default is to achieve all three comparsions listed as follow:\cr
#' If test_model contains "Both", the function compares both difference in group means and BPMV proportions.\cr
#' If test_model contains "Mean", the function compares difference in group means.\cr
#' If test_model contains "Pzero", the function compares difference in BPMV proportions.
#'
#' @details
#' The optimization function used here is \code{optim} with the method option as \code{"BFGS"}.
#'
#' If using "Fixed Difference" as the method to get detection limit, a small positive value is
#' recommended for \code{DL_value}.\cr
#' If using "Fixed Rate" as the method to get detection limit, a positive value larger than 1
#' is recommended for \code{DL_value}.\cr
#' If using "Fixed Value" as the method to get detection limit, a value smaller than the log value
#' of the minimum observation in the dataset is recommended for \code{DL_value}.
#'
#' This function requires at least one zero and one non-zero observations in each group.
#'
#' @return \item{feature_names}{A vector of feature names substracted from the input data.}
#' @return \item{pvalue_both}{A vector of estimated p-values for comparing both difference in
#' group means and BPMV proportions. pvalue_both will be NULL if test_model doesn't contain "Both".}
#' @return \item{pvalue_mean}{A vector of estimated p-values for comparing only difference
#' in group means. pvalue_mean will be NULL if test_model doesn't contain "Mean".}
#' @return \item{pvalue_zero}{A vector of estimated p-values for comparing only difference
#' in group BPMV proportions.pvalue_zero will be NULL if test_model doesn't contain "Pzero".}
#' @return \item{DL}{A vector of detection limits used.}
#' @return \item{estimates}{A matrix of estimates on optimization parameters, which can be
#' used to calculate group means, group BPMV proportions, and standard deviation.}
#'
#' @export
#' @seealso \code{\link{Getsample}}
#' @examples
#' #Get simulation samples#
#' data(simpool)
#' sim <- Getsample(numc=1000,numobs=100,pdiff=0.2,lfc=c(log(2),-log(2)),pzerodiff=NULL, simpool)
#'
#' data<- sim$simdata
#' paradata <- sim$Parameters
#' indall <- c()
#' for(i in 1:nrow(data)){
#'   ind <- (!all(data[i,][1:100]!=0)
#'           &!all(data[i,][1:100]==0)
#'           &!all(data[i,][101:200]!=0)
#'           &!all(data[i,][101:200]==0))
#'   indall<- c(indall, ind)
#' }
#' data_used <- data[indall,]
#'
#' example_result <- DASEV.DA(indata=data_used,cov.matrix=cbind(rep(1,200),c(rep(0,100),rep(1,100))),
#' test_cov= 1,test_cov_conti=NULL, min.non0n=3, requiredn=10, requiredn2=30,
#' DL_method= "Fixed Difference",DL_value=0.1,maxit_MLE=100,maxit=1000,
#' test_model=c("Both","Mean","Pzero"))

DASEV <- function(indata, cov.matrix, test_cov, test_cov_conti=NULL, min.non0n=3, requiredn=10,
                  requiredn2=30, DL_method= "Fixed Difference", DL_value=0.1,
                  maxit_MLE=100, maxit=10000, test_model=c("Both", "Mean", "Pzero")){
  if (!(DL_method == "Fixed Difference"|DL_method == "Fixed Rate"|DL_method ==
        "Fixed Value")){
    stop("Please enter the correct method to calculate the detect limit.")
  }

  if (is.na(DL_value)){
    stop("Please enter the correct value to calculate the detect limit.")
  }

  if (DL_method == "Fixed Rate" & DL_value < 1) {
    message("You are using the fixed rate method to calculate the detect limit,
            your input DL_value is less than 1, if this is not correct, please
            stop the program and check your input.")
  }

  if(min.non0n<3){
    stop("The minimum number of nonzero observation should be 3.")
  }

  #get data ready based on feature inclusion criteria
  n.ParamsF <- ncol(as.matrix(cov.matrix))
  n.ParamsN <- ncol(as.matrix(cov.matrix[, -test_cov]))
  ldata <- as.matrix(indata)
  idint<- rowSums(ldata> 0) >= min.non0n
  ldata <- ldata[idint,]

  check_cov <- length(test_cov) - length(test_cov_conti)
  if (length(test_cov_conti)>0){
    test_cov_dummy <- test_cov[-test_cov_conti]
  }else{
    test_cov_dummy <- test_cov
  }
  if (check_cov > 0){
    if(length(test_cov_dummy) > 1){
      test.cov <- cov.matrix[, test_cov_dummy]
      fslc <- function(test.cov){
        f.slc <- function(ldata){
          ind<- (!all(ldata[test.cov==0]!=0)
                 &!all(ldata[test.cov==0]==0)
                 &!all(ldata[test.cov==1]!=0)
                 &!all(ldata[test.cov==1]==0))
        }
        indall <-apply(ldata, 1, f.slc)
      }
      allind <- apply(test.cov, 2, fslc)
      allindsub <- rowSums(allind==TRUE)==ncol(allind)
    }else {
      f.slc <- function(ldata){
        ind<- (!all(ldata[test.cov==0]!=0)
               &!all(ldata[test.cov==0]==0)
               &!all(ldata[test.cov==1]!=0)
               &!all(ldata[test.cov==1]==0))
      }
      test.cov <- cov.matrix[, test_cov]
      allindsub <-apply(ldata, 1, f.slc)
    }
    ldata <- ldata[allindsub,]
  }

  ldata <- log(ldata)
  feature.names <- rownames(ldata)

  varprior <- c()
  for (i in 1:nrow(ldata)) {
    yvec <- ldata[i,]
    n <- (yvec > -Inf)
    covar.nonzero <- cov.matrix[n,]
    estbeta <- lm(yvec[n]~0+covar.nonzero)
    betasF <- coefficients(estbeta)
    betasF[is.na(betasF)] <- 0
    lsd <- log(sd(yvec[n]))
    if (DL_method == "Fixed Difference") {
      lod<-min(yvec[n])-DL_value
    } else if (DL_method == "Fixed Rate") {
      lod<-min(yvec[n])/DL_value
    } else if (DL_method == "Fixed Value") {
      lod <- DL_value
    } else {
      stop("Please enter the correct method to calculate the detect limit.")
    }

    mleest <- optim(par= c(betasF,lsd), fn=Getprior, yvec=yvec, lod=lod, covars=cov.matrix,
                    method="BFGS", control = list(maxit = maxit))
    if(mleest$convergence!=0)
      cat("\nGet prior distribution Convergence problems for feature ", i)
    varprior <- c(varprior, (exp(mleest$par[length(mleest$par)]))^2)
  }

  idint1<- rowSums(ldata> 0) >= requiredn
  varprior.sub <- varprior[idint1]
  count <- rowSums(ldata> -Inf)
  if(sum(idint1) < requiredn2){
    sort.varprior <- varprior[order(-count)]
    varprior.sub <- sort.varprior[1:requiredn2]
  }
  p.mean1 <- mean(varprior.sub)
  p.var1 <- var(varprior.sub)
  d0 <- 2*p.mean1^2/p.var1 + 4
  s0 <- sqrt(p.mean1*(d0-2)/d0)




  ################################################################################
  # Getting post statistics
  Para.Full <- c()
  MLE.Full <- c()
  MLE.Null <- c()
  MLE.Mean <- c()
  MLE.Pzero <- c()

  Pvalue <- c() #pvalue for Full model vs null model
  Pmean <-c() #pvalue for Full model vs same mean
  Pzero <-c() #pvalue for Full model vs same zero proportion

  DL <- c()
  for (i in 1:nrow(ldata)) {
    #print(i)
    yvec <- ldata[i,]
    n <- (yvec>-Inf)
    covar.nonzero <- cov.matrix[n,]
    if (sum(covar.nonzero[, 1] == covar.nonzero[, 2]) == nrow(covar.nonzero)){
      covar.nonzero[, 1]=1-covar.nonzero[, 1]
    }
    estbeta <- lm(yvec[n]~0+covar.nonzero)
    betasF <- coefficients(estbeta)
    betasF[is.na(betasF)] <- 0

    if (DL_method == "Fixed Difference") {
      lod<-min(yvec[n])-DL_value
    } else if (DL_method == "Fixed Rate") {
      lod<-min(yvec[n])/DL_value
    } else if (DL_method == "Fixed Value") {
      lod <- DL_value
    } else {
      stop("Please enter the correct method to calculate the detect limit.")
    }
    DL <- c(DL, lod)
    lsd0 <- log(sqrt(varprior[i]))# initial value for variance parameters
    mu <- mean(yvec[n])
    p <- 1 - length(yvec[n])/length(yvec)/(1-pnorm(lod, mu, exp(lsd0)))#true zero proportion starting value
    if (p>0){
      gamma0 <- log((1-p)/p)
    }else{
      gamma0 <- 1e+200
    }
    gammasF <-  c(gamma0, rep(0, ncol(cov.matrix)-1))

    covar.nonzero <- cov.matrix[n,]
    estbetaN <- lm(yvec[n]~0+covar.nonzero[, -test_cov])
    betasN <- coefficients(estbetaN)
    betasN[is.na(betasN)] <- 0
    gammasN <- gammasF[-test_cov]

    xF <- c(betasF, gammasF)#Full model params
    xN <- c(betasN, gammasN)#Null model params
    xM <- c(betasN, gammasF)#Params for model with different zero percentages but same means
    xP <- c(betasF, gammasN)#Params for model with different means but same zero percentages


    #Full Model
    nbeta <- length(betasF)
    Diff <- 1
    time <- 0
    lsd <- lsd0
    xFP <- xF
    while (Diff > 0.01 & time < maxit_MLE){
      time <- time +1
      outF <- optim(xFP, fn=loglikestep1, method = "BFGS", lsd=lsd, selectmodel="F",
                    lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                    cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
      xFP <- outF$par
      if(outF$convergence!=0)
        cat("\nStep 1 Get post statistics full model Convergence problems for feature ",i)
      out <- optim(lsd, fn=loglikestep2, selectmodel="F", method = "BFGS", betagamma=xFP,
                   lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                   cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
      Diff <- abs(out$par-lsd)
      lsd <- out$par
      if(out$convergence!=0)
        cat("\nStep 2 Get post statistics full model Convergence problems for feature ",i)
    }

    Para.Full <- rbind(Para.Full, c(outF$par, exp(out$par)))
    MLE.Full <- c(MLE.Full, outF$value)

    #Null model
    if ("Both" %in% test_model){
      nbeta <- length(betasN)
      Diff <- 1
      time <- 0
      lsd <- lsd0
      xNP <- xN
      while (Diff > 0.01 & time < maxit_MLE){
        time <- time +1
        outN <- optim(xNP, fn=loglikestep1, method = "BFGS", lsd=lsd, selectmodel="N",
                      lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                      cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        xNP <- outN$par
        if(outN$convergence!=0)
          cat("\nStep 1 Get post statistics null model Convergence problems for feature ",i)
        out <- optim(lsd, fn=loglikestep2, selectmodel="N", method = "BFGS", betagamma=xNP,
                     lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                     cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        Diff <- abs(out$par-lsd)
        lsd <- out$par
        if(out$convergence!=0)
          cat("\nStep 2 Get post statistics null model Convergence problems for feature ",i)

      }
      MLE.Null<-c(MLE.Null, outN$value)
    }else{
      MLE.Null <- c()
    }


    #Test mean
    if ("Mean" %in% test_model){
      nbeta <- length(betasN)
      Diff <- 1
      time <- 0
      lsd <- lsd0
      xMP <- xM
      while (Diff > 0.01 & time < maxit_MLE){
        time <- time +1
        outM <- optim(xMP, fn=loglikestep1, method = "BFGS", lsd=lsd, selectmodel="M",
                      lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                      cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        xMP <- outM$par
        if(outM$convergence!=0)
          cat("\nStep 1 Get post statistics mean model Convergence problems for feature ",i)
        out <- optim(lsd, fn=loglikestep2, selectmodel="M", method = "BFGS", betagamma=xMP,
                     lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                     cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        Diff <- abs(out$par-lsd)
        lsd <- out$par
        if(out$convergence!=0)
          cat("\nStep 2 Get post statistics mean model Convergence problems for feature ",i)

      }
      MLE.Mean <-c(MLE.Mean, outM$value)
    }else{
      MLE.Mean <- c()
    }


    #Test zero proportion
    if ("Pzero" %in% test_model){
      nbeta <- length(betasF)
      Diff <- 1
      time <- 0
      lsd <- lsd0
      xPP <- xP
      while (Diff > 0.01 & time < maxit_MLE){
        time <- time +1
        outP <- optim(xPP, fn=loglikestep1, method = "BFGS", lsd=lsd, selectmodel="P",
                      lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                      cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        xPP <- outP$par
        if(outP$convergence!=0)
          cat("\nStep 1 Get post statistics mean model Convergence problems for feature ",i)
        out <- optim(lsd, fn=loglikestep2, selectmodel="P", method = "BFGS", betagamma=xPP,
                     lod=lod, d0=d0, s0=s0, nbeta=nbeta, test_cov=test_cov,
                     cov.matrix=cov.matrix, yvec=yvec, control = list(maxit=maxit))
        Diff <- abs(out$par-lsd)
        lsd <- out$par
        if(out$convergence!=0)
          cat("\nStep 2 Get post statistics mean model Convergence problems for feature ",i)

      }
      MLE.Pzero <-c(MLE.Pzero, outP$value)
    }else{
      MLE.Pzero <- c()
    }
  }
  n.Params = ncol(cov.matrix)
  for (k in 1:n.Params){
    colnames(Para.Full)[k] <- paste("beta.", k, sep="")
  }
  for (k in (n.Params+1):(2*n.Params)){
    colnames(Para.Full)[k] <- paste("gamma.", k-n.Params, sep="")
  }
  colnames(Para.Full)[2*n.Params+1] <- "sd"

  if ("Both" %in% test_model){
    Pvalue <- c(Pvalue, 1-pchisq(2*(MLE.Null-MLE.Full), 2))
  }else{
    Pvalue <- c()
  }#Null vs Full
  if ("Mean" %in% test_model){
    Pmean <- c(Pmean, 1-pchisq(2*(MLE.Mean-MLE.Full), 1))
  }else{
    Pmean <- c()
  }#Mean vs Full
  if ("Pzero" %in% test_model){
    Pzero <- c(Pzero, 1-pchisq(2*(MLE.Pzero-MLE.Full), 1))
  }else{
    Pzero <- c()
  }#Pzero vs Full

  result <- list(feature_names =feature.names,
                 pvalue_both=Pvalue,
                 pvalue_mean=Pmean,
                 pvalue_zero=Pzero,
                 DL=DL,
                 estimates=Para.Full)

  return(result)

}
