#' Get simulated zero-infated dataset.
#'
#' This function returns a simulated dataset contains two groups with key parameters used in the simulation.
#'
#' @param numc Specifies the number of features. Defaults to 1000.
#' @param numobs Specifies the number of observations per group. Defaults to 100.
#' @param pdiff Specifies the percentage of differentially abundent features. Defaults to 0.2.
#' @param lfc The log fold change value as the difference in group means. Default to NULL.
#' The value can be a single number or a numeric vector. For example lfc=c(log(2),-log(2)).
#' @param pzerodiff The ratio for odds of BPMVs in two groups. Defaults to NULL.
#' The value can be a single number or a numeric vector. For example pzerodiff=c(0.5,2).
#' pzerodiff equals to (p1/(1-p1))/(p0(/(1-p0))).
#' @param simpool A data frame contains detection limit, mean, BPMV proportion, and standard
#' deviation. Simulation parameters are randomly sampled from this data frame.
#'
#' @details
#' This function assigns same number of observations for both groups.
#'
#' @return \item{simdata}{A data matrix for the simulated dataset.}
#' @return \item{Parameters}{A data matrix contains parameters used in the simulation.\cr
#' DL is the detection limit.\cr
#' mu0 is the group mean for group 0.\cr
#' mu1 is the group mean for group 1.\cr
#' p0 is the BPMV proportion for group 0.\cr
#' p1 is the BPMV proportion for group 1.\cr
#' sd is the standard deviation.}
#'
#' @export
#' @examples
#' #Get simulation samples#
#' data(simpool)
#' simdata1 <- Getsample(numc=1000,numobs=100,pdiff=0.2,lfc=c(log(2),-log(2)),pzerodiff=NULL, simpool)
#' simdata2 <- Getsample(numc=1000,numobs=100,pdiff=0.2,lfc=NULL,pzerodiff=c(0.5,2), simpool)

Getsample <- function(numc=1000,numobs=100,pdiff=0.2,lfc=NULL,pzerodiff=NULL,simpool){
  simint1 <- sample(row(simpool), size=numc, replace = T)
  simint2 <- sample(row(simpool), size=numc, replace = T)

  simlod <- simpool[simint1, 1]#Detect limit
  simmu <- simpool[simint1, 2] #Mean
  simt0p <- simpool[simint1, 3] #True zero proportion
  simsd <- simpool[simint2, 4] #Standard deviation
  if (length(lfc)>0){
    m.vector <- c(sample(lfc, size = numc*pdiff, replace = T), rep(0,round(numc*(1-pdiff))))
  }else{
    m.vector <- c(rep(0, numc))
  }

  if (length(pzerodiff)>0){
    p.vector <- c(sample(pzerodiff, size = numc*pdiff, replace = T),rep(1,round(numc*(1-pdiff))))
  }else{
    p.vector <- c(rep(1,numc))
  }
  m.case <- simmu+m.vector
  p.case <- p.case <- 1-(1/(1+p.vector*(simt0p/(1-simt0p))))


  sim.control <- data.frame(matrix(NA, nrow = numobs))
  sim.case <- data.frame(matrix(NA, nrow = numobs))

  for(i in 1:numc){
    p=simt0p[i]
    mu=simmu[i]
    lod=simlod[i]
    sd=simsd[i]
    pcase=p.case[i]
    mucase=m.case[i]

    if (mucase < lod){
      mucase=mucase+2*lfc}

    simcontrol <- Getfeature(p=p, mu=mu, sd=sd, lod=lod, numobs=numobs)
    simcase <- Getfeature(p=pcase, mu=mucase, sd=sd, lod=lod, numobs=numobs)
    sim.control <- cbind(sim.control, simcontrol)
    sim.case <- cbind(sim.case, simcase)
  }

  sim.control<- as.matrix(sim.control)
  sim.case <- as.matrix(sim.case)
  simall <- rbind(sim.control,sim.case)
  simall <- simall[, -1]
  simall <- t(as.matrix(simall))
  rownames(simall) <- paste("F",c(1:numc),sep="")
  colnames(simall) <- paste("Obs",c(1:(2*numobs)), sep="")

  paradata <- cbind(DL=simlod,mu0=simmu,
                    p0=simt0p,
                    sd=simsd,
                    mu1=m.case,
                    p1=p.case)
  rownames(paradata) <- paste("F",c(1:numc),sep="")
  result <- list(simdata=simall,
                 Parameters=paradata)
  return(result)
}


